# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RecommendationGetParams"]


class RecommendationGetParams(TypedDict, total=False):
    limit: int
    """The target size of the list of recommended tracks.

    For seeds with unusually small pools or when highly restrictive filtering is
    applied, it may be impossible to generate the requested number of recommended
    tracks. Debugging information for such cases is available in the response.
    Default: 20\\.. Minimum: 1\\.. Maximum: 100.
    """

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """

    max_acousticness: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_danceability: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_duration_ms: int
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_energy: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_instrumentalness: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_key: int
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_liveness: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_loudness: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_mode: int
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_popularity: int
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_speechiness: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_tempo: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_time_signature: int
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    max_valence: float
    """
    For each tunable track attribute, a hard ceiling on the selected track
    attribute’s value can be provided. See tunable track attributes below for the
    list of available options. For example, `max_instrumentalness=0.35` would filter
    out most tracks that are likely to be instrumental.
    """

    min_acousticness: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_danceability: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_duration_ms: int
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_energy: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_instrumentalness: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_key: int
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_liveness: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_loudness: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_mode: int
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_popularity: int
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_speechiness: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_tempo: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_time_signature: int
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    min_valence: float
    """
    For each tunable track attribute, a hard floor on the selected track attribute’s
    value can be provided. See tunable track attributes below for the list of
    available options. For example, `min_tempo=140` would restrict results to only
    those tracks with a tempo of greater than 140 beats per minute.
    """

    seed_artists: str
    """
    A comma separated list of
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for seed
    artists. Up to 5 seed values may be provided in any combination of
    `seed_artists`, `seed_tracks` and `seed_genres`.<br/> _**Note**: only required
    if `seed_genres` and `seed_tracks` are not set_.
    """

    seed_genres: str
    """
    A comma separated list of any genres in the set of
    [available genre seeds](/documentation/web-api/reference/get-recommendation-genres).
    Up to 5 seed values may be provided in any combination of `seed_artists`,
    `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
    and `seed_tracks` are not set_.
    """

    seed_tracks: str
    """
    A comma separated list of
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for a seed
    track. Up to 5 seed values may be provided in any combination of `seed_artists`,
    `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
    and `seed_genres` are not set_.
    """

    target_acousticness: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_danceability: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_duration_ms: int
    """Target duration of the track (ms)"""

    target_energy: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_instrumentalness: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_key: int
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_liveness: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_loudness: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_mode: int
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_popularity: int
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_speechiness: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_tempo: float
    """Target tempo (BPM)"""

    target_time_signature: int
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """

    target_valence: float
    """For each of the tunable track attributes (below) a target value may be provided.

    Tracks with the attribute values nearest to the target values will be preferred.
    For example, you might request `target_energy=0.6` and
    `target_danceability=0.8`. All target values will be weighed equally in ranking
    results.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel
from .image_object import ImageObject
from .copyright_object import CopyrightObject
from .external_url_object import ExternalURLObject

__all__ = ["ShowBase"]


class ShowBase(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the show.
    """

    available_markets: List[str]
    """
    A list of the countries in which the show can be played, identified by their
    [ISO 3166-1 alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
    """

    copyrights: List[CopyrightObject]
    """The copyright statements of the show."""

    description: str
    """A description of the show.

    HTML tags are stripped away from this field, use `html_description` field in
    case HTML tags are needed.
    """

    explicit: bool
    """
    Whether or not the show has explicit content (true = yes it does; false = no it
    does not OR unknown).
    """

    external_urls: ExternalURLObject
    """External URLs for this show."""

    href: str
    """A link to the Web API endpoint providing full details of the show."""

    html_description: str
    """A description of the show. This field may contain HTML tags."""

    images: List[ImageObject]
    """The cover art for the show in various sizes, widest first."""

    is_externally_hosted: bool
    """True if all of the shows episodes are hosted outside of Spotify's CDN.

    This field might be `null` in some cases.
    """

    languages: List[str]
    """
    A list of the languages used in the show, identified by their
    [ISO 639](https://en.wikipedia.org/wiki/ISO_639) code.
    """

    media_type: str
    """The media type of the show."""

    name: str
    """The name of the episode."""

    publisher: str
    """The publisher of the show."""

    total_episodes: int
    """The total number of episodes in the show."""

    type: Literal["show"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    show.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["TrackUpdateParams"]


class TrackUpdateParams(TypedDict, total=False):
    insert_before: int
    """
    The position where the items should be inserted.<br/>To reorder the items to the
    end of the playlist, simply set _insert_before_ to the position after the last
    item.<br/>Examples:<br/>To reorder the first item to the last position in a
    playlist with 10 items, set _range_start_ to 0, and _insert_before_
    to 10.<br/>To reorder the last item in a playlist with 10 items to the start of
    the playlist, set _range_start_ to 9, and _insert_before_ to 0.
    """

    range_length: int
    """The amount of items to be reordered.

    Defaults to 1 if not set.<br/>The range of items to be reordered begins from the
    _range_start_ position, and includes the _range_length_ subsequent
    items.<br/>Example:<br/>To move the items at index 9-10 to the start of the
    playlist, _range_start_ is set to 9, and _range_length_ is set to 2.
    """

    range_start: int
    """The position of the first item to be reordered."""

    snapshot_id: str
    """The playlist's snapshot ID against which you want to make the changes."""

    uris: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["TrackAddParams"]


class TrackAddParams(TypedDict, total=False):
    position: int
    """The position to insert the items, a zero-based index.

    For example, to insert the items in the first position: `position=0` ; to insert
    the items in the third position: `position=2`. If omitted, the items will be
    appended to the playlist. Items are added in the order they appear in the uris
    array. For example:
    `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M"], "position": 3}`
    """

    uris: SequenceNotStr[str]
    """
    A JSON array of the
    [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) to add. For
    example:
    `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M", "spotify:episode:512ojhOuo1ktJprKbVcKyQ"]}`<br/>A
    maximum of 100 items can be added in one request. _**Note**: if the `uris`
    parameter is present in the query string, any URIs listed here in the body will
    be ignored._
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FollowerFollowParams"]


class FollowerFollowParams(TypedDict, total=False):
    public: bool
    """Defaults to `true`.

    If `true` the playlist will be included in user's public playlists (added to
    profile), if `false` it will remain private. For more about public/private
    status, see [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .external_url_object import ExternalURLObject
from .linked_track_object import LinkedTrackObject
from .simplified_artist_object import SimplifiedArtistObject
from .track_restriction_object import TrackRestrictionObject

__all__ = ["SimplifiedTrackObject"]


class SimplifiedTrackObject(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """

    artists: Optional[List[SimplifiedArtistObject]] = None
    """The artists who performed the track.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    available_markets: Optional[List[str]] = None
    """
    A list of the countries in which the track can be played, identified by their
    [ISO 3166-1 alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
    """

    disc_number: Optional[int] = None
    """The disc number (usually `1` unless the album consists of more than one disc)."""

    duration_ms: Optional[int] = None
    """The track length in milliseconds."""

    explicit: Optional[bool] = None
    """
    Whether or not the track has explicit lyrics ( `true` = yes it does; `false` =
    no it does not OR unknown).
    """

    external_urls: Optional[ExternalURLObject] = None
    """External URLs for this track."""

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the track."""

    is_local: Optional[bool] = None
    """Whether or not the track is from a local file."""

    is_playable: Optional[bool] = None
    """
    Part of the response when
    [Track Relinking](/documentation/web-api/concepts/track-relinking/) is applied.
    If `true`, the track is playable in the given market. Otherwise `false`.
    """

    linked_from: Optional[LinkedTrackObject] = None
    """
    Part of the response when
    [Track Relinking](/documentation/web-api/concepts/track-relinking/) is applied
    and is only part of the response if the track linking, in fact, exists. The
    requested track has been replaced with a different track. The track in the
    `linked_from` object contains information about the originally requested track.
    """

    name: Optional[str] = None
    """The name of the track."""

    preview_url: Optional[str] = None
    """A URL to a 30 second preview (MP3 format) of the track."""

    restrictions: Optional[TrackRestrictionObject] = None
    """Included in the response when a content restriction is applied."""

    track_number: Optional[int] = None
    """The number of the track.

    If an album has several discs, the track number is the number on the specified
    disc.
    """

    type: Optional[str] = None
    """The object type: "track"."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .simplified_playlist_object import SimplifiedPlaylistObject

__all__ = ["PagingPlaylistObject"]


class PagingPlaylistObject(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[SimplifiedPlaylistObject]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..types import artist_top_tracks_params, artist_list_albums_params, artist_bulk_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.artist_object import ArtistObject
from ..types.artist_top_tracks_response import ArtistTopTracksResponse
from ..types.artist_list_albums_response import ArtistListAlbumsResponse
from ..types.artist_bulk_retrieve_response import ArtistBulkRetrieveResponse
from ..types.artist_list_related_artists_response import ArtistListRelatedArtistsResponse

__all__ = ["ArtistsResource", "AsyncArtistsResource"]


class ArtistsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ArtistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return ArtistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArtistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return ArtistsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistObject:
        """
        Get Spotify catalog information for a single artist identified by their unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/artists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ArtistObject,
        )

    def bulk_retrieve(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistBulkRetrieveResponse:
        """
        Get Spotify catalog information for several artists based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the artists.
              Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/artists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, artist_bulk_retrieve_params.ArtistBulkRetrieveParams),
            ),
            cast_to=ArtistBulkRetrieveResponse,
        )

    def list_albums(
        self,
        id: str,
        *,
        include_groups: str | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[ArtistListAlbumsResponse]:
        """
        Get Spotify catalog information about an artist's albums.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          include_groups: A comma-separated list of keywords that will be used to filter the response. If
              not supplied, all album types will be returned. <br/> Valid values are:<br/>-
              `album`<br/>- `single`<br/>- `appears_on`<br/>- `compilation`<br/>For example:
              `include_groups=album,single`.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/artists/{id}/albums",
            page=SyncCursorURLPage[ArtistListAlbumsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_groups": include_groups,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    artist_list_albums_params.ArtistListAlbumsParams,
                ),
            ),
            model=ArtistListAlbumsResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def list_related_artists(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistListRelatedArtistsResponse:
        """
        Get Spotify catalog information about artists similar to a given artist.
        Similarity is based on analysis of the Spotify community's listening history.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/artists/{id}/related-artists",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ArtistListRelatedArtistsResponse,
        )

    def top_tracks(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistTopTracksResponse:
        """
        Get Spotify catalog information about an artist's top tracks by country.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/artists/{id}/top-tracks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"market": market}, artist_top_tracks_params.ArtistTopTracksParams),
            ),
            cast_to=ArtistTopTracksResponse,
        )


class AsyncArtistsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncArtistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncArtistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArtistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncArtistsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistObject:
        """
        Get Spotify catalog information for a single artist identified by their unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/artists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ArtistObject,
        )

    async def bulk_retrieve(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistBulkRetrieveResponse:
        """
        Get Spotify catalog information for several artists based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the artists.
              Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/artists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, artist_bulk_retrieve_params.ArtistBulkRetrieveParams),
            ),
            cast_to=ArtistBulkRetrieveResponse,
        )

    def list_albums(
        self,
        id: str,
        *,
        include_groups: str | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ArtistListAlbumsResponse, AsyncCursorURLPage[ArtistListAlbumsResponse]]:
        """
        Get Spotify catalog information about an artist's albums.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          include_groups: A comma-separated list of keywords that will be used to filter the response. If
              not supplied, all album types will be returned. <br/> Valid values are:<br/>-
              `album`<br/>- `single`<br/>- `appears_on`<br/>- `compilation`<br/>For example:
              `include_groups=album,single`.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/artists/{id}/albums",
            page=AsyncCursorURLPage[ArtistListAlbumsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_groups": include_groups,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    artist_list_albums_params.ArtistListAlbumsParams,
                ),
            ),
            model=ArtistListAlbumsResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def list_related_artists(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistListRelatedArtistsResponse:
        """
        Get Spotify catalog information about artists similar to a given artist.
        Similarity is based on analysis of the Spotify community's listening history.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/artists/{id}/related-artists",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ArtistListRelatedArtistsResponse,
        )

    async def top_tracks(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ArtistTopTracksResponse:
        """
        Get Spotify catalog information about an artist's top tracks by country.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              artist.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/artists/{id}/top-tracks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"market": market}, artist_top_tracks_params.ArtistTopTracksParams),
            ),
            cast_to=ArtistTopTracksResponse,
        )


class ArtistsResourceWithRawResponse:
    def __init__(self, artists: ArtistsResource) -> None:
        self._artists = artists

        self.retrieve = to_raw_response_wrapper(
            artists.retrieve,
        )
        self.bulk_retrieve = to_raw_response_wrapper(
            artists.bulk_retrieve,
        )
        self.list_albums = to_raw_response_wrapper(
            artists.list_albums,
        )
        self.list_related_artists = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                artists.list_related_artists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.top_tracks = to_raw_response_wrapper(
            artists.top_tracks,
        )


class AsyncArtistsResourceWithRawResponse:
    def __init__(self, artists: AsyncArtistsResource) -> None:
        self._artists = artists

        self.retrieve = async_to_raw_response_wrapper(
            artists.retrieve,
        )
        self.bulk_retrieve = async_to_raw_response_wrapper(
            artists.bulk_retrieve,
        )
        self.list_albums = async_to_raw_response_wrapper(
            artists.list_albums,
        )
        self.list_related_artists = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                artists.list_related_artists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.top_tracks = async_to_raw_response_wrapper(
            artists.top_tracks,
        )


class ArtistsResourceWithStreamingResponse:
    def __init__(self, artists: ArtistsResource) -> None:
        self._artists = artists

        self.retrieve = to_streamed_response_wrapper(
            artists.retrieve,
        )
        self.bulk_retrieve = to_streamed_response_wrapper(
            artists.bulk_retrieve,
        )
        self.list_albums = to_streamed_response_wrapper(
            artists.list_albums,
        )
        self.list_related_artists = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                artists.list_related_artists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.top_tracks = to_streamed_response_wrapper(
            artists.top_tracks,
        )


class AsyncArtistsResourceWithStreamingResponse:
    def __init__(self, artists: AsyncArtistsResource) -> None:
        self._artists = artists

        self.retrieve = async_to_streamed_response_wrapper(
            artists.retrieve,
        )
        self.bulk_retrieve = async_to_streamed_response_wrapper(
            artists.bulk_retrieve,
        )
        self.list_albums = async_to_streamed_response_wrapper(
            artists.list_albums,
        )
        self.list_related_artists = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                artists.list_related_artists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.top_tracks = async_to_streamed_response_wrapper(
            artists.top_tracks,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["RecommendationListAvailableGenreSeedsResponse"]


class RecommendationListAvailableGenreSeedsResponse(BaseModel):
    genres: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import track_retrieve_params, track_bulk_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.shared.track_object import TrackObject
from ..types.track_bulk_retrieve_response import TrackBulkRetrieveResponse

__all__ = ["TracksResource", "AsyncTracksResource"]


class TracksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TracksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return TracksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TracksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return TracksResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackObject:
        """
        Get Spotify catalog information for a single track identified by its unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/tracks/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"market": market}, track_retrieve_params.TrackRetrieveParams),
            ),
            cast_to=TrackObject,
        )

    def bulk_retrieve(
        self,
        *,
        ids: str,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackBulkRetrieveResponse:
        """
        Get Spotify catalog information for multiple tracks based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=4iV5W9uYEdYUVa79Axb7Rh,1301WleyT98MSxVHPZCA6M`. Maximum: 50 IDs.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/tracks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "market": market,
                    },
                    track_bulk_retrieve_params.TrackBulkRetrieveParams,
                ),
            ),
            cast_to=TrackBulkRetrieveResponse,
        )


class AsyncTracksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTracksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTracksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTracksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncTracksResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackObject:
        """
        Get Spotify catalog information for a single track identified by its unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/tracks/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"market": market}, track_retrieve_params.TrackRetrieveParams),
            ),
            cast_to=TrackObject,
        )

    async def bulk_retrieve(
        self,
        *,
        ids: str,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackBulkRetrieveResponse:
        """
        Get Spotify catalog information for multiple tracks based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=4iV5W9uYEdYUVa79Axb7Rh,1301WleyT98MSxVHPZCA6M`. Maximum: 50 IDs.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/tracks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ids": ids,
                        "market": market,
                    },
                    track_bulk_retrieve_params.TrackBulkRetrieveParams,
                ),
            ),
            cast_to=TrackBulkRetrieveResponse,
        )


class TracksResourceWithRawResponse:
    def __init__(self, tracks: TracksResource) -> None:
        self._tracks = tracks

        self.retrieve = to_raw_response_wrapper(
            tracks.retrieve,
        )
        self.bulk_retrieve = to_raw_response_wrapper(
            tracks.bulk_retrieve,
        )


class AsyncTracksResourceWithRawResponse:
    def __init__(self, tracks: AsyncTracksResource) -> None:
        self._tracks = tracks

        self.retrieve = async_to_raw_response_wrapper(
            tracks.retrieve,
        )
        self.bulk_retrieve = async_to_raw_response_wrapper(
            tracks.bulk_retrieve,
        )


class TracksResourceWithStreamingResponse:
    def __init__(self, tracks: TracksResource) -> None:
        self._tracks = tracks

        self.retrieve = to_streamed_response_wrapper(
            tracks.retrieve,
        )
        self.bulk_retrieve = to_streamed_response_wrapper(
            tracks.bulk_retrieve,
        )


class AsyncTracksResourceWithStreamingResponse:
    def __init__(self, tracks: AsyncTracksResource) -> None:
        self._tracks = tracks

        self.retrieve = async_to_streamed_response_wrapper(
            tracks.retrieve,
        )
        self.bulk_retrieve = async_to_streamed_response_wrapper(
            tracks.bulk_retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ...types import browse_get_new_releases_params, browse_get_featured_playlists_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.browse_get_new_releases_response import BrowseGetNewReleasesResponse
from ...types.browse_get_featured_playlists_response import BrowseGetFeaturedPlaylistsResponse

__all__ = ["BrowseResource", "AsyncBrowseResource"]


class BrowseResource(SyncAPIResource):
    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> BrowseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return BrowseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrowseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return BrowseResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def get_featured_playlists(
        self,
        *,
        limit: int | Omit = omit,
        locale: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrowseGetFeaturedPlaylistsResponse:
        """
        Get a list of Spotify featured playlists (shown, for example, on a Spotify
        player's 'Browse' tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/browse/featured-playlists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "locale": locale,
                        "offset": offset,
                    },
                    browse_get_featured_playlists_params.BrowseGetFeaturedPlaylistsParams,
                ),
            ),
            cast_to=BrowseGetFeaturedPlaylistsResponse,
        )

    def get_new_releases(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrowseGetNewReleasesResponse:
        """
        Get a list of new album releases featured in Spotify (shown, for example, on a
        Spotify player’s “Browse” tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/browse/new-releases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    browse_get_new_releases_params.BrowseGetNewReleasesParams,
                ),
            ),
            cast_to=BrowseGetNewReleasesResponse,
        )


class AsyncBrowseResource(AsyncAPIResource):
    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBrowseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncBrowseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrowseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncBrowseResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def get_featured_playlists(
        self,
        *,
        limit: int | Omit = omit,
        locale: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrowseGetFeaturedPlaylistsResponse:
        """
        Get a list of Spotify featured playlists (shown, for example, on a Spotify
        player's 'Browse' tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/browse/featured-playlists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "locale": locale,
                        "offset": offset,
                    },
                    browse_get_featured_playlists_params.BrowseGetFeaturedPlaylistsParams,
                ),
            ),
            cast_to=BrowseGetFeaturedPlaylistsResponse,
        )

    async def get_new_releases(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrowseGetNewReleasesResponse:
        """
        Get a list of new album releases featured in Spotify (shown, for example, on a
        Spotify player’s “Browse” tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/browse/new-releases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    browse_get_new_releases_params.BrowseGetNewReleasesParams,
                ),
            ),
            cast_to=BrowseGetNewReleasesResponse,
        )


class BrowseResourceWithRawResponse:
    def __init__(self, browse: BrowseResource) -> None:
        self._browse = browse

        self.get_featured_playlists = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                browse.get_featured_playlists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_new_releases = to_raw_response_wrapper(
            browse.get_new_releases,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._browse.categories)


class AsyncBrowseResourceWithRawResponse:
    def __init__(self, browse: AsyncBrowseResource) -> None:
        self._browse = browse

        self.get_featured_playlists = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                browse.get_featured_playlists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_new_releases = async_to_raw_response_wrapper(
            browse.get_new_releases,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._browse.categories)


class BrowseResourceWithStreamingResponse:
    def __init__(self, browse: BrowseResource) -> None:
        self._browse = browse

        self.get_featured_playlists = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                browse.get_featured_playlists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_new_releases = to_streamed_response_wrapper(
            browse.get_new_releases,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._browse.categories)


class AsyncBrowseResourceWithStreamingResponse:
    def __init__(self, browse: AsyncBrowseResource) -> None:
        self._browse = browse

        self.get_featured_playlists = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                browse.get_featured_playlists,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_new_releases = async_to_streamed_response_wrapper(
            browse.get_new_releases,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._browse.categories)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .image_object import ImageObject
from .followers_object import FollowersObject
from .external_url_object import ExternalURLObject

__all__ = ["ArtistObject"]


class ArtistObject(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    artist.
    """

    external_urls: Optional[ExternalURLObject] = None
    """Known external URLs for this artist."""

    followers: Optional[FollowersObject] = None
    """Information about the followers of the artist."""

    genres: Optional[List[str]] = None
    """A list of the genres the artist is associated with.

    If not yet classified, the array is empty.
    """

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the artist."""

    images: Optional[List[ImageObject]] = None
    """Images of the artist in various sizes, widest first."""

    name: Optional[str] = None
    """The name of the artist."""

    popularity: Optional[int] = None
    """The popularity of the artist.

    The value will be between 0 and 100, with 100 being the most popular. The
    artist's popularity is calculated from the popularity of all the artist's
    tracks.
    """

    type: Optional[Literal["artist"]] = None
    """The object type."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    artist.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["PlayerStartPlaybackParams"]


class PlayerStartPlaybackParams(TypedDict, total=False):
    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """

    context_uri: str
    """Optional.

    Spotify URI of the context to play. Valid contexts are albums, artists &
    playlists. `{context_uri:"spotify:album:1Je1IMUlBXcx1Fz0WE7oPT"}`
    """

    offset: Dict[str, object]
    """Optional.

    Indicates from where in the context playback should start. Only available when
    context_uri corresponds to an album or playlist object "position" is zero based
    and can’t be negative. Example: `"offset": {"position": 5}` "uri" is a string
    representing the uri of the item to start at. Example:
    `"offset": {"uri": "spotify:track:1301WleyT98MSxVHPZCA6M"}`
    """

    position_ms: int
    """Indicates from what position to start playback.

    Must be a positive number. Passing in a position that is greater than the length
    of the track will cause the player to start playing the next song.
    """

    uris: SequenceNotStr[str]
    """Optional.

    A JSON array of the Spotify track URIs to play. For example:
    `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh", "spotify:track:1301WleyT98MSxVHPZCA6M"]}`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from .tracks import (
    TracksResource,
    AsyncTracksResource,
    TracksResourceWithRawResponse,
    AsyncTracksResourceWithRawResponse,
    TracksResourceWithStreamingResponse,
    AsyncTracksResourceWithStreamingResponse,
)
from ...types import playlist_update_params, playlist_retrieve_params
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .followers import (
    FollowersResource,
    AsyncFollowersResource,
    FollowersResourceWithRawResponse,
    AsyncFollowersResourceWithRawResponse,
    FollowersResourceWithStreamingResponse,
    AsyncFollowersResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.playlist_retrieve_response import PlaylistRetrieveResponse

__all__ = ["PlaylistsResource", "AsyncPlaylistsResource"]


class PlaylistsResource(SyncAPIResource):
    @cached_property
    def tracks(self) -> TracksResource:
        return TracksResource(self._client)

    @cached_property
    def followers(self) -> FollowersResource:
        return FollowersResource(self._client)

    @cached_property
    def images(self) -> ImagesResource:
        return ImagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> PlaylistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return PlaylistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlaylistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return PlaylistsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        playlist_id: str,
        *,
        additional_types: str | Omit = omit,
        fields: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaylistRetrieveResponse:
        """
        Get a playlist owned by a Spotify user.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          fields: Filters for the query: a comma-separated list of the fields to return. If
              omitted, all fields are returned. For example, to get just the playlist''s
              description and URI: `fields=description,uri`. A dot separator can be used to
              specify non-reoccurring fields, while parentheses can be used to specify
              reoccurring fields within objects. For example, to get just the added date and
              user ID of the adder: `fields=tracks.items(added_at,added_by.id)`. Use multiple
              parentheses to drill down into nested objects, for example:
              `fields=tracks.items(track(name,href,album(name,href)))`. Fields can be excluded
              by prefixing them with an exclamation mark, for example:
              `fields=tracks.items(track(name,href,album(!name,href)))`

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._get(
            f"/playlists/{playlist_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional_types": additional_types,
                        "fields": fields,
                        "market": market,
                    },
                    playlist_retrieve_params.PlaylistRetrieveParams,
                ),
            ),
            cast_to=PlaylistRetrieveResponse,
        )

    def update(
        self,
        playlist_id: str,
        *,
        collaborative: bool | Omit = omit,
        description: str | Omit = omit,
        name: str | Omit = omit,
        public: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Change a playlist's name and public/private state.

        (The user must, of course,
        own the playlist.)

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          collaborative: If `true`, the playlist will become collaborative and other users will be able
              to modify the playlist in their Spotify client. <br/> _**Note**: You can only
              set `collaborative` to `true` on non-public playlists._

          description: Value for playlist description as displayed in Spotify Clients and in the Web
              API.

          name: The new name for the playlist, for example `"My New Playlist Title"`

          public: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/playlists/{playlist_id}",
            body=maybe_transform(
                {
                    "collaborative": collaborative,
                    "description": description,
                    "name": name,
                    "public": public,
                },
                playlist_update_params.PlaylistUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPlaylistsResource(AsyncAPIResource):
    @cached_property
    def tracks(self) -> AsyncTracksResource:
        return AsyncTracksResource(self._client)

    @cached_property
    def followers(self) -> AsyncFollowersResource:
        return AsyncFollowersResource(self._client)

    @cached_property
    def images(self) -> AsyncImagesResource:
        return AsyncImagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPlaylistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncPlaylistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlaylistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncPlaylistsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        playlist_id: str,
        *,
        additional_types: str | Omit = omit,
        fields: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaylistRetrieveResponse:
        """
        Get a playlist owned by a Spotify user.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          fields: Filters for the query: a comma-separated list of the fields to return. If
              omitted, all fields are returned. For example, to get just the playlist''s
              description and URI: `fields=description,uri`. A dot separator can be used to
              specify non-reoccurring fields, while parentheses can be used to specify
              reoccurring fields within objects. For example, to get just the added date and
              user ID of the adder: `fields=tracks.items(added_at,added_by.id)`. Use multiple
              parentheses to drill down into nested objects, for example:
              `fields=tracks.items(track(name,href,album(name,href)))`. Fields can be excluded
              by prefixing them with an exclamation mark, for example:
              `fields=tracks.items(track(name,href,album(!name,href)))`

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._get(
            f"/playlists/{playlist_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "additional_types": additional_types,
                        "fields": fields,
                        "market": market,
                    },
                    playlist_retrieve_params.PlaylistRetrieveParams,
                ),
            ),
            cast_to=PlaylistRetrieveResponse,
        )

    async def update(
        self,
        playlist_id: str,
        *,
        collaborative: bool | Omit = omit,
        description: str | Omit = omit,
        name: str | Omit = omit,
        public: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Change a playlist's name and public/private state.

        (The user must, of course,
        own the playlist.)

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          collaborative: If `true`, the playlist will become collaborative and other users will be able
              to modify the playlist in their Spotify client. <br/> _**Note**: You can only
              set `collaborative` to `true` on non-public playlists._

          description: Value for playlist description as displayed in Spotify Clients and in the Web
              API.

          name: The new name for the playlist, for example `"My New Playlist Title"`

          public: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/playlists/{playlist_id}",
            body=await async_maybe_transform(
                {
                    "collaborative": collaborative,
                    "description": description,
                    "name": name,
                    "public": public,
                },
                playlist_update_params.PlaylistUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PlaylistsResourceWithRawResponse:
    def __init__(self, playlists: PlaylistsResource) -> None:
        self._playlists = playlists

        self.retrieve = to_raw_response_wrapper(
            playlists.retrieve,
        )
        self.update = to_raw_response_wrapper(
            playlists.update,
        )

    @cached_property
    def tracks(self) -> TracksResourceWithRawResponse:
        return TracksResourceWithRawResponse(self._playlists.tracks)

    @cached_property
    def followers(self) -> FollowersResourceWithRawResponse:
        return FollowersResourceWithRawResponse(self._playlists.followers)

    @cached_property
    def images(self) -> ImagesResourceWithRawResponse:
        return ImagesResourceWithRawResponse(self._playlists.images)


class AsyncPlaylistsResourceWithRawResponse:
    def __init__(self, playlists: AsyncPlaylistsResource) -> None:
        self._playlists = playlists

        self.retrieve = async_to_raw_response_wrapper(
            playlists.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            playlists.update,
        )

    @cached_property
    def tracks(self) -> AsyncTracksResourceWithRawResponse:
        return AsyncTracksResourceWithRawResponse(self._playlists.tracks)

    @cached_property
    def followers(self) -> AsyncFollowersResourceWithRawResponse:
        return AsyncFollowersResourceWithRawResponse(self._playlists.followers)

    @cached_property
    def images(self) -> AsyncImagesResourceWithRawResponse:
        return AsyncImagesResourceWithRawResponse(self._playlists.images)


class PlaylistsResourceWithStreamingResponse:
    def __init__(self, playlists: PlaylistsResource) -> None:
        self._playlists = playlists

        self.retrieve = to_streamed_response_wrapper(
            playlists.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            playlists.update,
        )

    @cached_property
    def tracks(self) -> TracksResourceWithStreamingResponse:
        return TracksResourceWithStreamingResponse(self._playlists.tracks)

    @cached_property
    def followers(self) -> FollowersResourceWithStreamingResponse:
        return FollowersResourceWithStreamingResponse(self._playlists.followers)

    @cached_property
    def images(self) -> ImagesResourceWithStreamingResponse:
        return ImagesResourceWithStreamingResponse(self._playlists.images)


class AsyncPlaylistsResourceWithStreamingResponse:
    def __init__(self, playlists: AsyncPlaylistsResource) -> None:
        self._playlists = playlists

        self.retrieve = async_to_streamed_response_wrapper(
            playlists.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            playlists.update,
        )

    @cached_property
    def tracks(self) -> AsyncTracksResourceWithStreamingResponse:
        return AsyncTracksResourceWithStreamingResponse(self._playlists.tracks)

    @cached_property
    def followers(self) -> AsyncFollowersResourceWithStreamingResponse:
        return AsyncFollowersResourceWithStreamingResponse(self._playlists.followers)

    @cached_property
    def images(self) -> AsyncImagesResourceWithStreamingResponse:
        return AsyncImagesResourceWithStreamingResponse(self._playlists.images)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.track_object import TrackObject

__all__ = ["TrackBulkRetrieveResponse"]


class TrackBulkRetrieveResponse(BaseModel):
    tracks: List[TrackObject]
